<?php
namespace Modules\Fornecimento\Modules\AutorizacaoFornecimento\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Modules\Fornecimento\Modules\AutorizacaoFornecimento\Services\AprovacaoAlcadasService;

class AprovacaoAlcadasController
{
    public function __construct(AprovacaoAlcadasService $service)
    {
        $this->service = $service;
    }
    
    public function getAlcadasToAprroval(Request\Filter $request, Response $response)
    {
        try {
            $filterCriteria = $request->getFilterCriteria();
            $conditions     = self::getConditionsFromFilter($filterCriteria);
            
            $conditions['P_DTINCLUSAO_INI'] = $conditions['DTINCLUSAO']['start'];
            $conditions['P_DTINCLUSAO_FIN'] = $conditions['DTINCLUSAO']['end'];
            
            if (empty($conditions['DTINCLUSAO']['start']) && empty($conditions['DTINCLUSAO']['end'])) {
                $conditions['P_TDTINCLUSAO'] = 'T';
            } else {
                $conditions['P_TDTINCLUSAO'] = '';
            }
            
            $conditions['P_DTENTREGAAF_INI'] = $conditions['DTENTREGAAF']['start'];
            $conditions['P_DTENTREGAAF_FIN'] = $conditions['DTENTREGAAF']['end'];
            
            if (empty($conditions['DTENTREGAAF']['start']) && empty($conditions['DTENTREGAAF']['end'])) {
                $conditions['P_TDTENTREGAAF'] = 'T';
            } else {
                $conditions['P_TDTENTREGAAF'] = '';
            }
            
            $alcadas        = $this->service->getAlcadas($conditions);
            $dataset        = self::formatAlcadasToResponse($alcadas);
            
            $response->addDataSet(new DataSet('alcadas', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getConditionsFromFilter($filter)
    {
        $conditions = array();
        
        foreach ($filter->getConditions() as $filter) {
            if (is_array($filter['value']) && strpos($filter['columnName'], 'DT') != false) 
                $conditions[$filter['columnName']] = implode("_", $filter['value']) . '_';
            else 
                $conditions[$filter['columnName']] = $filter['value'];
        }
        
        return $conditions;
    }
    
    public function formatAlcadasToResponse($alcadas)
    {
        $formattedAlcada = array();
        
        foreach ($alcadas as $key=>$alcada) {
            $column = $this->service->defineColumn($alcada['IDAPROVADO']);
            $label = $alcada['NRDEMANDA'].' - '.$alcada['NMFILDEST'];
            $alcada = self::concatFilialData($alcada);
            $alcada = self::applyMaskOnFields($alcada);
            $description =  'AF: '.$alcada['NRDEMANDA'].
                            '<br>Entrega: '.explode(' ', $alcada['DTPROGENTAF'])[0].
                            '<br>Fornecedor: '.$alcada['NMFANTFORN'].
                            '<br>Prioridade: '.$alcada['IDCLASSIFIC'].
                            '<br>Total AF: '.$alcada['VRTOTALAF'].
                            '<br>Alçada: '.$alcada['NMALCADA'].
                            '<br>';
                            
            $formattedAlcada[$key] = self::arrayMount($column, $label, $description, $alcada);
        }
        
        return $formattedAlcada;
    }
    
    public function arrayMount($column, $label, $description, $alcada)
    {
        $alcada["column"]      = $column;
        $alcada["label"]       = $label;
        $alcada["movable"]     = false;
        $alcada["description"] = $description;
        
        return $alcada;
    }
    
    public function concatFilialData($dataset)
    {
        $dataset['CDNMFILDEST'] = $dataset['CDFILDEST'].' - '.$dataset['NMFILDEST'];
        
        return $dataset;
    }
    
    public function applyMaskOnFields($alcada)
    {
        $alcada['IDCLASSIFIC'] = $this->service->getNameClassific($alcada['IDCLASSIFIC']);
        $alcada['IDAPROVADO']  = $this->service->getNameIdAprovado($alcada['IDAPROVADO']);
        $alcada['VRAPROVCMP']  = $alcada["IDTIPOVRAPROV"] == "VALOR" ? self::currencyMaskApply($alcada['VRAPROVCMP']) : self::percentMaskApply($alcada['VRAPROVCMP']);
        $alcada['VRTOTALAF']   = self::currencyMaskApply($alcada['VRTOTALAF']);
        
        return $alcada;
    }
    
    public function currencyMaskApply($value)
    {
        return 'R$' . number_format($value, 2, ',', '.');
    }
    
    public function percentMaskApply($value)
    {
        return number_format($value, 2, ',', '.') . '%';
    }
}
